/**
 * AbstractGebuehrenTatbestand.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.io.Serializable;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Eine Oberklasse, um GebührenTatbestände zu speichern, entweder aus dem VV RVG
 * oder dem KV GKG. Als Oberklasse für alle Gebührentatbestände nur mit
 * beschreibung und Verweis auf die Klasse der Gebührenordnung. Abgeleitete
 * Klassen können den eigentlichen Inhalt wie Gebührensatz, Auslagenhöhe oder
 * Mehrwertsteuersatz bereitstellen.
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 21.08.2014
 */
abstract class GebuehrenTatbestand implements Serializable {

	private static final long serialVersionUID = -7148753363484353653L;
	private/* final */String bezeichnung;
	private/* final */Class<? extends GebuehrenTabelle> gebuehrenKlasse;

	/**
	 * Erstellt eine Instanz der Klasse mit den angegebenen Werten.
	 * 
	 * @param bezeichnung
	 *            Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *            Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param gebuehrenKlasse
	 *            Klasse der Gebühr, also den Verweis auf die Klasse eines
	 *            Ablegers von GebuehrenTabelle, also entweder
	 *            GerichtsGebuehrenTabelle.class oder
	 *            AnwaltsGebuehrenTabelle.class
	 */
	public GebuehrenTatbestand(String bezeichnung,
			Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		super();
		this.bezeichnung = bezeichnung;
		this.gebuehrenKlasse = gebuehrenKlasse;
	}

	/**
	 * Konstruktor ohne Parameter für {@link JAXB}
	 * 
	 */
	public GebuehrenTatbestand() {
		super();
	}

	/**
	 * Die Methode gibt die Bezeichnung des Tatbestands als String zurück.
	 * Dieser sollte den jeweiligen Tatbestand einigermaßen eindeutig
	 * beschreiben.
	 * 
	 * @return Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *         Verzeichnis als String, z.B. "Nr. 1100 KV"
	 */
	@XmlAttribute(name = "bezeichnung")
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * @param bezeichnung d. {@link #bezeichnung}, d. gesetzt werden soll als String
	 */
	public void setBezeichnung(String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	/**
	 * @return gebuehrenKlasse gibt zurück, auf welche Gebührentabelle sich der
	 *         Tatbestand bezieht, also eine Klasse, die GebuehrenTabelle
	 *         erweitert.
	 */
	@XmlAttribute(name = "gesetz")
	public Class<? extends GebuehrenTabelle> getGebuehrenKlasse() {
		return gebuehrenKlasse;
	}

	/**
	 * @param gebuehrenKlasse d. {@link #gebuehrenKlasse}, d. gesetzt werden soll als Class<? extends GebuehrenRechner>
	 */
	public void setGebuehrenKlasse(Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		this.gebuehrenKlasse = gebuehrenKlasse;
	}

}