/**
 * HauptklageForderung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Klasse für Hauptklageforderungen im Gegensatz zu Hilfsklageforderungen
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date: 31.07.2014
 *
 */
class KlageForderung extends Forderung {
	private static final long serialVersionUID = 1087072460398990941L;
	private static final String BESCHREIBUNG = "Hauptklageforderung";
	/**
	 * Der boolean mit dem Namen wirtschaftlichIdentisch gibt an, ob die
	 * Hauptklageforderung wirtschaftlich identisch zu der jeweils in der Liste
	 * vorangehenden Klageforderung ist.
	 */
	private boolean wirtschaftlichIdentisch;

	/**
	 * Konstruktor:
	 * 
	 * @param wert
	 *            die Höhe der Forderung, also ihren Wert.
	 * @param erfolg
	 *            inwieweit die Forderung Erfolg hatte.
	 * @param wirtschaftlichIdentisch
	 *            ob die Hauptklageforderung wirtschaftlich identisch zu der
	 *            jeweils in der Liste vorangehenden Klageforderung ist.
	 * 
	 * @throws IllegalArgumentException
	 *             wenn der Erfolg höher als der Wert ist
	 */
	public KlageForderung(double wert, double erfolg,
			boolean wirtschaftlichIdentisch) throws IllegalArgumentException {
		super(false, wert, erfolg);
		setWirtschaftlichIdentisch(wirtschaftlichIdentisch);
	}

	/**
	 * Der Standardkonstruktor belegt vor, dass es sich nicht um eine
	 * wirtschaftlich identische Klageforderung handelt.
	 * 
	 * @param wert
	 *            die Höhe der Forderung, also ihren Wert.
	 * @param erfolg
	 *            inwieweit die Forderung Erfolg hatte.
	 * 
	 * @throws IllegalArgumentException
	 *             wenn der Erfolg höher als der Wert ist
	 */
	public KlageForderung(double wert, double erfolg)
			throws IllegalArgumentException {
		this(wert, erfolg, false);
	}

	/**
	 * Konstruktor ohne Paramter für {@link JAXB}
	 * 
	 */
	public KlageForderung() {
		super();
		super.setGegenforderung(false);
	}

	/**
	 * Gibt eine Beschreibung der Forderungsart zurück
	 * 
	 * @return "Hauptklageforderung"
	 * 
	 * @see eu.gronos.kostenrechner.Forderung#getBeschreibung()
	 */
	@Override
	String getBeschreibung() {
		return BESCHREIBUNG;
	}

	/**
	 * @return the wirtschaftlichIdentisch
	 */
	@XmlAttribute(name = "p45i3GKG")
	public boolean isWirtschaftlichIdentisch() {
		return wirtschaftlichIdentisch;
	}

	/**
	 * @param wirtschaftlichIdentisch
	 *            the wirtschaftlichIdentisch to set
	 */
	public void setWirtschaftlichIdentisch(boolean wirtschaftlichIdentisch) {
		this.wirtschaftlichIdentisch = wirtschaftlichIdentisch;
	}

}
