package eu.gronos.kostenrechner;

import java.io.InputStream;
import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;

/**
 * Testfälle für die Tenorierungs-Klassen
 * 
 * @author Peter Schuster (setrok)
 * @date 12.01.2014
 * 
 */
public class KostenrechnerTest {

	/**
	 * main:
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		Beteiligter kl = new Beteiligter(Beteiligter.KLAEGER,
				Beteiligter.MAENNLICH);
		Beteiligter bk = new Beteiligter(Beteiligter.BEKLAGTE,
				Beteiligter.WEIBLICH);
		Beteiligter dwbk = new Beteiligter(Beteiligter.DRITTWIDERBEKLAGTE,
				Beteiligter.WEIBLICH);

		// "Für %,d,00 EUR beträgt 1,0 Gerichtsgebühr %,.2f EUR und 1,0 RA-Gebühr %,.2f EUR.%n"
		System.out.println("Test für die Gebührenberechnung:");
		GebuehrenTabelle rvg = new AnwaltsGebuehrenTabelle();
		GebuehrenTabelle gkg = new GerichtsGebuehrenTabelle();

		final KostenrechnerTest instance = new KostenrechnerTest();
		InputStream resourceAsStream = instance.getClass().getClassLoader()
				.getResourceAsStream("resources/rvg.xml");
		rvg = new AnwaltsGebuehrenTabelle(resourceAsStream);
		// rvg = new AnwaltsGebuehrenTabelle("resources/rvg.xml");
		resourceAsStream = instance.getClass().getClassLoader()
				.getResourceAsStream("resources/gkg.xml");
		gkg = new GerichtsGebuehrenTabelle(resourceAsStream);

		// final ArrayList<Long> streitwertliste =
		// rvg.errechneStreitwertListe();
		// for (long l : streitwertliste)
		// System.out.printf("%d,00 EUR, ", l);
		// System.out.printf("gesamt %d Einträge.%n%n", streitwertliste.size());
		// for (long streitwert : new long[] { 500L, 1000L, 1500L, 2000L, 3000L,
		// 4000L, 5000L, 6000L, 7000L, 8000L, 9000L, 10000L, 13000L,
		// 16000L, 19000L, 22000L, 25000L, 30000L, 35000L, 40000L, 45000L,
		// 50000L, 65000L, 80000L, 95000L, 110000L, 125000L, 140000L,
		// 155000L, 170000L, 185000L, 200000L, 230000L, 260000L, 290000L,
		// 320000L, 350000L, 380000L, 410000L, 440000L, 470000L, 500000 })
		System.out.println("Streitwert bis \t| GKG 1,0 \t| RVG 1,0");
		System.out.println("-------------- \t| ------- \t| -------");
		for (long streitwert : rvg.errechneStreitwertListe())
			System.out.printf("%,d,00 EUR \t| %,.2f EUR \t| %,.2f EUR.%n",
					streitwert, gkg.errechneGebuehr(streitwert),
					rvg.errechneGebuehr(streitwert));
		System.out.println();

		// Vorbereitungen zur Teilklagerücknahme Anders/Gehle, A-198
		final GebuehrenSatzTatbestand[] vorher = new GebuehrenSatzTatbestand[] {
				GerichtsGebuehrenTabelle.KV1210,
				AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100 };
		final GebuehrenSatzTatbestand[] nachher = new GebuehrenSatzTatbestand[] {
				AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104 };
		StreitwertReduktion swr;

		// Mehrkostenmethode
		System.out.println("\nMehrkostenmethode, Anders/Gehle, A-198:");
		swr = new MehrkostenMethode(kl, bk, vorher, nachher, new long[] {
				10000L, 5000L }, 5000L, true);
		System.out.println(swr.erzeugeTenor());

		// Quotenmethode
		System.out.println("\nQuotenmethode, Anders/Gehle, A-198:");
		swr = new QuotenMethode(kl, bk, vorher, nachher, new long[] { 10000L,
				5000L }, 5000L, true);
		System.out.println(swr.erzeugeTenor());

		// Baumbach'sche Formel
		System.out
				.println("Baumbach'sche Formel mit gesamtschuldnerischer Verurteilung, Anders/Gehle, Rn. A-204");
		BaumbachBeteiligtenListe bbtl = new BaumbachBeteiligtenListe();
		bbtl.add(new BaumbachBeteiligter(kl, 0.0, 0.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 10000.0, 0.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 10000.0, 0.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 10000.0, 0.0, false));
		bbtl.add(new BaumbachGesamtschuldnerschaft(Beteiligter.BEKLAGTE,
				Beteiligter.WEIBLICH + Beteiligter.PLURAL, 10000.0, false,
				new int[] { 1, 2 }, bbtl));
		BaumbachFormel bf = new BaumbachFormel(bbtl, true, false);
		System.out.println(bf.erzeugeTenor());

		// Test mit BaumbachGesamtschuldnern
		System.out
				.println("Teilweise Gesamtschuld und Einzelverurteilung, Anders/Gehle, Rn. A-205");
		bbtl = new BaumbachBeteiligtenListe();
		bbtl.add(new BaumbachBeteiligter(kl, 0.0, 0.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 10000.0, 2500.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 10000.0, 0.0, false));
		bbtl.add(new BaumbachGesamtschuldnerschaft(Beteiligter.BEKLAGTE,
				Beteiligter.WEIBLICH + Beteiligter.PLURAL, 5000.0, false,
				new int[] { 1, 2 }, bbtl));
		bf = new BaumbachFormel(bbtl, true, false);
		System.out.println(bf.erzeugeTenor());

		// Widerklage eines Streitgenossen
		System.out
				.println("Widerklage eines Streitgenossen, Anders/Gehle, Rn. A-207");
		System.out.println();
		/*
		 * Hier komme ich zu einem anderen Ergebnis als Anders/Gehle, aber ich
		 * habe glaub ich recht.
		 */
		bbtl = new BaumbachBeteiligtenListe();
		bbtl.add(new BaumbachBeteiligter(kl, 7500.0, 4500.0, true));
		bbtl.add(new BaumbachBeteiligter(bk, 2500.0, 0.0, true));
		bbtl.add(new BaumbachBeteiligter(bk, 2500.0, 0.0, false));
		bbtl.add(new BaumbachGesamtschuldnerschaft(Beteiligter.BEKLAGTE,
				Beteiligter.WEIBLICH + Beteiligter.PLURAL, 1000.0, false,
				new int[] { 1, 2 }, bbtl));
		bf = new BaumbachFormel(bbtl, true, false);
		System.out.println(bf.erzeugeTenor());

		// Test mit Drittwiderbeklagten
		System.out
				.println("Widerklage und Drittwiderklage, Theimer/Theimer, Muster 87, III. 2. Beispiel 7");
		bbtl = new BaumbachBeteiligtenListe();
		bbtl.add(new BaumbachBeteiligter(kl, 900.0, 0.0, true));
		bbtl.add(new BaumbachBeteiligter(dwbk, 900.0, 0.0, true));
		bbtl.add(new BaumbachGesamtschuldnerschaft(
				Beteiligter.DRITTWIDERBEKLAGTE, Beteiligter.WEIBLICH
						+ Beteiligter.PLURAL, 450, true, new int[] { 0, 1 },
				bbtl));
		bbtl.add(new BaumbachBeteiligter(bk, 1200.0, 0.0, false));
		bbtl.add(new BaumbachBeteiligter(bk, 1200.0, 0.0, true));
		bbtl.add(new BaumbachGesamtschuldnerschaft(Beteiligter.BEKLAGTE,
				Beteiligter.WEIBLICH + Beteiligter.PLURAL, 300.0, false,
				new int[] { 3, 4 }, bbtl));
		bf = new BaumbachFormel(bbtl, true, false);
		System.out.println(bf.erzeugeTenor());

		// Test der Hilfsaufrechnungs-Fälle
		ArrayList<Forderung> alf = new ArrayList<Forderung>();
		AngriffsVerteidigungsmittelStaffelung avs;
		// 1
		System.out.println("Testfall 1: Anders/Gehle G-22, Grundfall");
		alf.clear();
		alf.add(new KlageForderung(10000.0, 10000.0));
		alf.add(new AufrechnungsForderung(15000.0, 15000.0));
		avs = new AngriffsVerteidigungsmittelStaffelung(alf, kl, bk);
		System.out.println(avs.erzeugeTenor());
		System.out.println();
		// 2
		System.out.println("Testfall 2: Anders/Gehle G-22, Abwandlung");
		alf.clear();
		alf.add(new KlageForderung(10000.0, 7500.0));
		alf.add(new AufrechnungsForderung(15000.0, 4000.0));
		avs = new AngriffsVerteidigungsmittelStaffelung(alf, kl, bk);
		System.out.println(avs.erzeugeTenor());
		System.out.println();
		// 3
		System.out.println("Testfall 3: Anders/Gehle G-23:");
		alf.clear();
		alf.add(new KlageForderung(10000.0, 7500.0));
		alf.add(new AufrechnungsForderung(7500.0, 5000.0));
		alf.add(new AufrechnungsForderung(2500.0, 1000.0));
		avs = new AngriffsVerteidigungsmittelStaffelung(alf, kl, bk);
		System.out.println(avs.erzeugeTenor());
		System.out.println();

		JAXBContext context;
		Marshaller marshaller;
		try {
			context = JAXBContext.newInstance(GebuehrenTabelle.class);
			marshaller = context.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
					Boolean.TRUE);
			marshaller.marshal(rvg, System.out);
			marshaller.marshal(gkg, System.out);
		} catch (PropertyException e1) {
			// TODO Auto-generated catch block
			System.err.println(e1.getLocalizedMessage());
			e1.printStackTrace();
		} catch (JAXBException e2) {
			// TODO Auto-generated catch block
			System.err.println(e2.getLocalizedMessage());
			e2.printStackTrace();
		}

	}
}
